"""
Classifies: CHEBI:142348 hexahydronaphthalenes
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetSymmSSSR

def is_hexahydronaphthalenes(smiles: str):
    """
    Determines if a molecule is a hexahydronaphthalene or derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hexahydronaphthalene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Get ring information
    rings = mol.GetRingInfo()
    if rings.NumRings() < 2:
        return False, "Molecule does not contain bicyclic system"
        
    # Find fused 6-membered rings
    fused_rings = []
    ring_atoms = rings.AtomRings()
    
    # Check for pairs of 6-membered rings
    for i, ring1 in enumerate(ring_atoms):
        if len(ring1) != 6:
            continue
        for j, ring2 in enumerate(ring_atoms[i+1:], i+1):
            if len(ring2) != 6:
                continue
                
            # Check if rings share exactly 2 atoms (fused)
            shared = set(ring1).intersection(set(ring2))
            if len(shared) == 2:
                fused_rings.append((ring1, ring2))
                
    if not fused_rings:
        return False, "No fused 6-membered ring system found"
        
    # For each fused ring pair, check if at least one ring has significant saturation
    for ring1, ring2 in fused_rings:
        ring1_atoms = [mol.GetAtomWithIdx(i) for i in ring1]
        ring2_atoms = [mol.GetAtomWithIdx(i) for i in ring2]
        
        # Count number of sp3 carbons in each ring
        ring1_sp3 = sum(1 for atom in ring1_atoms if atom.GetHybridization() == Chem.HybridizationType.SP3)
        ring2_sp3 = sum(1 for atom in ring2_atoms if atom.GetHybridization() == Chem.HybridizationType.SP3)
        
        # Check if at least one ring has significant saturation (4 or more sp3)
        if ring1_sp3 >= 4 or ring2_sp3 >= 4:
            # Get substituents
            all_ring_atoms = set(ring1).union(set(ring2))
            substituents = []
            
            for atom_idx in all_ring_atoms:
                atom = mol.GetAtomWithIdx(atom_idx)
                for neighbor in atom.GetNeighbors():
                    if neighbor.GetIdx() not in all_ring_atoms:
                        substituents.append(neighbor.GetSymbol())
                        
            if len(substituents) > 0:
                return True, f"Substituted hexahydronaphthalene with substituents: {', '.join(sorted(set(substituents)))}"
            else:
                return True, "Unsubstituted hexahydronaphthalene"
                
    return False, "No hexahydronaphthalene core structure found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:142348',
                          'name': 'hexahydronaphthalenes',
                          'definition': 'Any carbobycyclic compound that is an '
                                        'hexahydronaphthalene or a compound '
                                        'obtained from an hexahydronaphthalene '
                                        'by formal substitution of one or more '
                                        'hydrogens.',
                          'parents': ['CHEBI:36785']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.019417475728155338 is too low.\n'
               'True positives: '
               "[('[H][C@@]12C[C@H](C)C[C@H](C(O)=O)[C@@]1([H])[C@](C)(C(=O)CCO)C([C@H](C)CCCC)=C(O)C2=O', "
               "'Substituted hexahydronaphthalene with substituents: C, O')]\n"
               'False positives: '
               "[('O=C1C(=O)C2=C([C@]34C(=O)C=5C=C(C)C=C(C5C([C@@]4(C2)[C@@H](O)[C@H](O)C=C3)=O)O)C(=C1C[C@H]6C(=C)CC[C@@H]7[C@@]6(CC[C@H](C7(C)C)O)C)O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C[C@@H]([C@H]1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C)C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('C[C@H](C[C@H]([C@@H](C(C)(C)O[C@H]1[C@@H]([C@H]([C@@H]([C@@H](CO)O1)O)O)O)O)O)[C@]2(CC[C@@]3(C)[C@@]4(CC=C5[C@](CC[C@@H](C5(C)C)O[C@H]6[C@@H](C([C@@H]([C@@H](CO[C@H]7[C@@H]([C@H]([C@@H]([C@@H](CO)O7)O)O)O)O6)O)O)O)([H])[C@]4(C)CC[C@]23C)[H])[H]', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C(O[C@H]1[C@@]2([C@]3(C=CC(C)=C[C@H]3O[C@H](C1)[C@]24OC4)C)C)/C=C\\\\C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]12CC[C@@]3(C)[C@]4([H])CCC5=C(C(=O)OC5)[C@@]4(C)[C@H](O)C[C@]3([H])[C@@]1(C)CCC[C@@]2(C)CO', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('OC(=O)C(F)(F)F.[H][C@]12C[C@H](O)CC[C@@]11CCN2Cc2cc(O)c(OC)cc12', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]12CC=C3C[C@@H](O)CC[C@]3(C)[C@@]1([H])CC[C@]1(C)[C@H](CC[C@@]21[H])C(C)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('[H][C@]12CC[C@]3([H])C(C)(C)[C@H](O)CC[C@@]3(C)[C@]1([H])C(=O)C=C(C=C)[C@H]2C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C1[C@@]2([C@@H](C(=C)CC1)C[C@@H](C(O)(C)C)CC2)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C1[C@@]([C@@]2([C@@](C1)(C3=CC[C@@]4([H])[C@H](C)[C@@H](O)CC[C@]4(C)[C@]3(CC2)[H])[H])C)([C@H](C)CC/C(=C/C)/C(C)C)[H]', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C(O[C@H]1[C@@]2([C@@]3([C@@H](C=C(C)CC3)O[C@H]([C@@H]1O)[C@]24OC4)C)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]12CCC(=C)[C@H](C[C@H](O)[C@](C)(O)C=C)[C@@]1(C)CCC[C@]2(C)C(O)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O(C1CC=2C(C3C(C4C(C(CC4)C(CCCC(C)C)C)(CC3)C)CC2)(CC1)C)C(=O)CCCCCCCCC', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('CC(=O)[C@H]1CC[C@H]2[C@@H]3CC[C@H]4CC(=O)CC[C@@H]4[C@H]3CC[C@]12C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O(C1C(C2C(C3C(C4(C(C5C(C(O)C4)(CCC(C5)(C)C)C(O)=O)=CC3)C)(CC2)C)(CC1)C)(C)C)C6OC(C(O)C(O)C6O)COC7OCC(O)C(O)C7O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O[C@]12[C@@]([C@](CC1)([C@@](O)([C@H](O)CCC(C)C)C)[H])(CC[C@@]3([C@@]4([C@@](C[C@@H](O[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O)CO)[C@@H](O)C4)(C(=O)C=C23)[H])C)[H])C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O1C2C(C3C(CC2)(C(C(CC3)=C)CC4=C(O)C(=C(OC4=O)C)C)C)(CCC(OC(=O)/C=C/C=C/C=C/C(CC)C)C1(C)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O(C1CC=2C(C3C(C4C(C(CC4)C(CC/C(/C(C)C)=C\\\\C)C)(CC3)C)CC2)(CC1)C)C5OC(C(O)C(O)C5O)COC(=O)CCCCCCCCCCCCCCC', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1C=C[C@@]23[C@H](O)C[C@@H](CC2[C@@]1(CCC(=O)NC4=C(O)C(C(=O)OC)=CC=C4O)C)C(C3)=C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('CC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCC(=O)O[C@H]1CC[C@]2(C)[C@H]3CC[C@]4(C)[C@H](CC[C@H]4[C@@H]3CC=C2C1)[C@H](C)CCCC(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O1C2C34C5(OC6C2C(C1C3=C(C(=O)C5(O)C7C4C(=O)C89C(C(NC8=O)CC(C)C)C(C(=CC9C=C(CCC(=O)C7=O)C)C)C)C)C(=O)C%10%11C(C(NC%10=O)CC(C)C)C(C(=CC%11C=C(CCC6O)C)C)C)O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O1[C@H]2O[C@@H]3[C@]4([C@@H](C5=C(C(C)C)C[C@@H]([C@]5(C)CC4)O)CC=C6[C@H]3[C@@]2(O)[C@]7(O[C@@H]6O[C@]7(C1)O)O)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])CCC4=CC(=O)C=C[C@]4(C)[C@@]3([H])CC[C@]12C)[C@H](C)\\\\C=C\\\\CC(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('C1C[C@]2([C@]([C@]([C@H]1O)(C)CO)(CC[C@@]3([C@@]2(CC=C4[C@]3(CC[C@@]5([C@]4(CC(CC5)(C)C)[H])C([O-])=O)C)[H])C)[H])C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1N(C(CC(O)(C(=O)O)C(CC)C)C(C1=C(O)/C=C/[C@@H]2[C@@H]3[C@@H](C=C[C@@H]2C)CCCC3)=O)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('[H][C@@]12CCC3=CC(=O)C=C[C@]3(C)[C@@]1([H])[C@@H](O)C[C@@]1(C)[C@@]2([H])C[C@H]2OC(CCC)O[C@@]12C(=O)CO', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C1OCC=2C1(O)[C@@]3([C@H](C(CCC3)(C)C)[C@@H](C2)OC(=O)CCCCC)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C(O)[C@@]1([C@@H](O)CC[C@]2([C@H]1CCC3=C2C=C4C5=C(C=CC=C5)N(C4=C3)C6=CC7=C(NC8=C7C=C9[C@@]%10([C@H]([C@](C(=O)O)([C@@H](O)CC%10)C)CCC9=C8)C)C=C6)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])CCC4=CC(=O)CC[C@]4(C)[C@@]3([H])CC[C@]12C)[C@H](C)[C@@H](O)[C@H](O)[C@@H](C)C(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('C[C@@H]1C[C@H]2CC3=C(CCC(=O)N3)[C@@]3(C1)[C@@H]2CCCN3C', "
               "'Substituted hexahydronaphthalene with substituents: C, N'), "
               "('[H][C@]12C(C)(C)CCC[C@@]11C(=O)O[C@]2(O)C(=O)C2=C[C@](C)(C[C@@H](O)[C@]12O)C=C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('F[C@@]12[C@]([C@]3([C@@](C[C@@H]1O)(C)[C@](CC3)(C(COC(C)=O)=O)OC(C)=O)[H])(C[C@H](C=4[C@]2(C)C=C(C(C4)=O)Br)F)[H]', "
               "'Substituted hexahydronaphthalene with substituents: F, C, "
               "O'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])[C@H](O)C=C4C[C@@H](O)CC[C@]4(C)[C@@]3([H])CC[C@]12C)[C@H](C)CCCC(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@]12C[C@@]3([H])C(=COC[C@@]3([H])[C@]([H])(Cc3c1n(C)c1cc4O[C@]5(C)OC[C@@]6([H])[C@]7([H])Cc8c(n(C)c9ccccc89)[C@]([H])(C[C@@]6([H])[C@]5([H])Cc4cc31)N7C)N2C)C(C)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O[C@@H]1[C@@]([C@]2([C@@]([C@@]3([C@]([C@]4(C([C@]5([C@@](CC4)(CCC(C5)(C)C)C(O)=O)[H])=CC3)C)(CC2)C)[H])(C[C@@H]1O)C)[H])(C)C(O)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1O[C@@H](C)[C@@]2([C@@]13C(=C)[C@](C[C@@H]4[C@@]3(CC[C@@H]([C@]4(CCC(=O)OC)C)C(O)(C)C)C)(C)C2=O)O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O1[C@H]2C(=CC[C@H]3[C@H]2[C@]4(OC(C)(C)O[C@H]4C[C@@H]3C)[C@@H](C1)CO)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C/C=C\\\\1/CN2CC[C@@]34C5=CC=CC=C5N6[C@]4(C(C=C([C@H]7C[C@@]89C%10=CC=CC=C%10N%11C(C=C[C@@]([C@]/%12(C[C@@]8(N7C\\\\C%12=C\\\\C)[H])[H])([C@@]9%11[H])[H])=O)C6=O)[C@]1(C[C@@]32[H])[H])[H]', "
               "'Substituted hexahydronaphthalene with substituents: C, O, "
               "N'), "
               "('Cl[C@H]1[C@](C=C)([C@]2([N+]#[C-])[C@]3(O)C=4C=5C(=CC=CC5C([C@H]3C1)(C)C)NC4C([C@H]([C@@H]2O)O)(C)C)C', "
               "'Substituted hexahydronaphthalene with substituents: Cl, C, O, "
               "N'), "
               "('O[C@]1([C@H]2[C@@]([C@@H]3C(C[C@](CCO)(C)C[C@H]3O)=CC2)(CCC1)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O[C@@H]1CC=2[C@@]([C@@]3([C@]([C@]4([C@@]([C@@]([C@@H]([C@@]5([C@](C5)([C@@H](C(C)C)C)C)[H])C)(CC4)[H])(CC3)C)[H])(CC2)[H])[H])(CC1)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O1C2C(C3(C(C4C(CC3)C5(C(=CC4)CC(O)CC5O[C@@H]6OC[C@H](OC(=O)C(O)C(CC)C)[C@H](O)[C@H]6O[C@@H]7O[C@H]([C@H](O)[C@@H](O)[C@H]7O)C)C)C2)C)C(C18OCC(CC8)=C)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O[C@H]1C[C@]2([C@]3([C@@]([C@@]4([C@@](CC3)(C([C@@H](O)CC4)(C)C)[H])C)(CC[C@@]2([C@]5([C@@]1([C@@H](O)C=C([C@H]5C)C)C)[H])[H])[H])C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('COC(=O)[C@@]1(C)CC[C@@]2(CC[C@]3(C)C(=CC[C@@H]4[C@@]5(C)C[C@H](O)[C@H](O[C@@H]6OC[C@@H](O)[C@H](O)[C@H]6O)[C@@](C)(CO)[C@@H]5CC[C@@]34C)[C@@H]2C1)C(O)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O[C@@]1([C@@]2([C@]([C@]3([C@@]([C@@]4(C(CC3)=CC(=O)CC4)C)(CC2)[H])[H])(CC1)[H])C)CC', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('Cl[C@H]1C[C@]2([C@]3([C@@]([C@H]([C@@H](C3)C)C(=O)C)(CC[C@@]2([C@@]4(C1=CC(=O)CC4)C)[H])C)[H])[H]', "
               "'Substituted hexahydronaphthalene with substituents: Cl, C'), "
               "('O=C1OCC=2C1=C(O)C3=C(O[C@@]4(CC[C@@H]5[C@@]([C@H]4C3)(CC[C@H](C5(C)C)O)C)C)C2C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1[C@@H]([C@]2(CC=C3C([C@@]2(C1)C)=CC[C@@H]4[C@@]3(CC[C@@H](C4(C)C)O)C)C)[C@H](C(=O)O)CCC=C(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C1C2([C@@]3([C@@H](C=C(C)[C@H](C3)O)O[C@H](C1)[C@@]24OC4)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('OC1C(C2C(C3C(C4(C(=C5C(CC4)(CCC(C5)(C)C(OC)=O)C)C=C3)C)(CC2)C)(CC1)C)(CO)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('C1[C@@]2([C@]3(CC[C@]4([C@]([C@@]3(CC=C2C[C@H](C1)O)[H])(CC[C@@]4([C@H](C)CCCC(C)C)[H])[H])C)[H])C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('C(COC(CC)=O)(=O)[C@]1(OC(CC)=O)[C@]2(C)[C@@](C[C@H]1C)([C@]3([C@@]([C@H](C2)O)(F)[C@]4(C)C(CC3)=CC(C=C4)=O)[H])[H]', "
               "'Substituted hexahydronaphthalene with substituents: F, C, "
               "O'), "
               "('[H][C@@]12[C@H](O)[C@H](OC(C)=O)[C@@]3(C)O[C@](C)(CC(=O)[C@]3([H])[C@@]1(C)[C@@H](O)CCC2(C)C)C=C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O1[C@@]2([C@@]([C@@]3([C@](O)([C@]4([C@](CC3)([C@@]5(C(=CC4)C[C@@H](O)CC5)C)[H])[H])C2)C)([C@@H]([C@]16OC[C@@H](CC6)C)C)[H])[H]', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])CCC4=CC(=O)CC[C@]4(C)[C@@]3([H])CC[C@]12C)[C@@H](C)O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('[H][C@@]1(C[C@]2([H])N(CCc3c2[nH]c2ccccc32)C[C@@H]1CC)C(=C/OC)\\\\C(=O)OC', "
               "'Substituted hexahydronaphthalene with substituents: C, N'), "
               "('[H][C@@]1(C[C@]1([H])[C@@H](C)[C@@]1([H])CC[C@]2([H])[C@]1(C)CC[C@]1([H])[C@@]3(C)CC[C@H](O)C[C@@]33OO[C@@]21C=C3)[C@H](C)C(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O[C@@H]1[C@H]([C@]2([C@@](C3=C(C4[C@@]([C@](CC4)([C@@H](CCC=C(C)C)C)[H])(CC3)C)CC2)(CC1)C)[H])C(O)=O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C(O)[C@@H]([C@@H]1[C@@]2([C@@](C3=C([C@@]4([C@H](C([C@H](OC(=O)C[C@@](O)(CC(=O)OC)C)CC4)(C)C)CC3)C)CC2)(C)[C@H](C1)O)C)CCC=C(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C[C@@]12CC[C@H]3C(=CC[C@H]4C(C)(C)[C@H](O)CC[C@]34C)[C@@]1(C)CC[C@H]2C1COC(=O)C1', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[C@@H]1(C[C@H](C([C@]2([H])[C@]1(C)[C@@]3(C(C=C([C@H]([C@]3(CC2)[H])C)C=C)=O)[H])(C)C)O)O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C1OCC=2[C@@]1(O)[C@@]3([C@H](C(CCC3)(C)C)[C@@H](C2)OC(=O)/C=C/C=C/CC(O)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@]1(OC[C@@H](O)[C@H](O)[C@H]1O)O[C@@H]1[C@@H](O)[C@H](O)[C@@H](CO)O[C@@]1([H])O[C@@H]1[C@@H](O)[C@@H](O)[C@@H](C)O[C@@]1([H])O[C@H]1CC[C@@]2(C)[C@@]([H])(CC[C@]3(C)[C@]2([H])C=C[C@]24OC[C@@]5(CC[C@@H](C)[C@H](C)[C@@]25[H])[C@H](O)C[C@@]34C)C1(C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O[C@@H]1CC=2[C@@](C3C(C4[C@@]([C@](CC4)([C@H](C)C(=O)CCC(C)C)[H])(CC3)C)CC2)(CC1)C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('ClC1=C2C(=CC3=C1NC=4C=CC=CC34)[C@@]5([C@H]([C@](C(=O)O)([C@@H](O)CC5)C)CC2)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('S(OC1=CC2=C([C@@]3([C@]([C@]4([C@](CC3)(CCC4)C)[H])(CC2)[H])[H])C=C1)(O)(=O)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1OCC23C4(C5(OC5)C(OC2C=C(C)CC3)C(C4OC(=O)C=CC=CC(OCCC(=C1)C)C(O)C)O)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]12CCC3=CC(=O)CC[C@]3(C)[C@@]1([H])CC[C@@]1(C)[C@@]2([H])CC[C@]1(O)[C@@H](C)O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1C2=C(O)C=C(OC)C=C2C(=O)[C@@]3([C@]1(O)[C@H](O[C@@H](C3)CC(=O)N)CCC)O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('CC(C)C1=C[C@@]23CC[C@@H]4[C@@](CCCC4(C)C)(C(=O)O2)C3=CC1=O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C[C@@]12[C@]3(CC[C@]1([C@]4([C@](CC2)([C@]5(C)C(C=C4)=CC(CC5)=O)[H])[H])[H])OC(CC3)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O[C@@H]1[C@]2(C(=C3[C@@]([C@@]4([C@](CC3)(C[C@H](O)CC4)[H])C)(C1)[H])[C@H](O)C[C@@]2([C@@H](CCC(O)=O)C)[H])C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@]1(OC[C@@H](O[C@]2([H])O[C@H](CO)[C@H](O)[C@H](O)[C@H]2O)[C@H](O)[C@H]1O)O[C@H]1[C@H](C)O[C@@]([H])(O[C@@H]2[C@@H](O)[C@@H](OC(=O)\\\\C=C\\\\c3ccc(OC)cc3)[C@@H](C)O[C@H]2OC(=O)[C@]23CCC(C)(C)C[C@@]2([H])C2=CC[C@]4([H])[C@@]5(C)C[C@H](O)[C@H](O[C@]6([H])O[C@H](CO)[C@@H](O)[C@H](O)[C@H]6O)[C@@](C)(C(O)=O)[C@]5([H])CC[C@@]4(C)[C@]2(CO)CC3)[C@H](O)[C@@H]1O[C@]1([H])OC[C@](O)(CO)[C@H]1O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1N[C@@](OC)(CC(C)C)C=C1C(=O)[C@H]2[C@H](C(=C[C@@]3([C@@H]2CC[C@@H](C3)C)C)C)/C(=C/C)/C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('[H][C@@]12CC(C)(C)CC[C@@]1(CC[C@]1(C)C2=CC[C@]2([H])[C@@]3(C)CC[C@H](O[C@@H]4O[C@@H]([C@@H](O)[C@H](O[C@@H]5O[C@@H](C)[C@H](O)[C@@H](O)[C@H]5O)[C@H]4O)C(O)=O)[C@@](C)(C=O)[C@]3([H])CC[C@@]12C)C(O)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C1[C@](OC[C@@]2([C@]1([C@@H]3[C@H](C[C@@H](C)CC3)C=C2)C)O)(O)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('OC1C(C2C3C(C4(C(C5(C(CC4)C(C(=O)CC5)(C)C)C)CC3)C)(CCC2(CC1)C)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C1=CC(C=C2[C@]1([C@@]3([C@@](C[C@@]2(F)[H])([C@]4([C@@](C[C@@]3(O)[H])(C)[C@]5([C@@](C4)(OC(O5)(C)C)[H])C(=O)COC(C)=O)[H])[H])F)C)=O', "
               "'Substituted hexahydronaphthalene with substituents: F, C, "
               "O'), "
               "('O1[C@@]2(OC(C)(C)[C@H]([C@H]2O)C)[C@H]([C@@H]3[C@@]4([C@@]1(C5=C(C(=C(O)C=C5)C)CC4)CC3)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])C=CC4=CC(=O)CC[C@]4(C)[C@@]3([H])C[C@H](O)[C@]12C)[C@H](C)CCC([O-])=O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('Oc1ccc2C[C@H]3N(CC4CC4)CC[C@@]45[C@@H](Oc1c24)c1[nH]c2ccccc2c1C[C@@]35O', "
               "'Substituted hexahydronaphthalene with substituents: C, O, "
               "N'), "
               "('O=C1O[C@@H]2[C@H]3O[C@@]3(C=O)C([C@@]4([C@@H]2[C@@]1(CCC4)C)C)=CC(=O)OC', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@@]12CC[C@](O)(C(=O)CO)[C@@]1(C)C[C@H](O)[C@@]1([H])[C@@]2([H])C[C@@H](O)C2=CC(=O)C=C[C@]12C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('CC(C(=O)SCCNC(=O)CCNC(=O)[C@H](O)C(C)(C)COP([O-])(=O)OP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP([O-])([O-])=O)n1cnc2c(N)ncnc12)=C1CC[C@H]2[C@@H]3CCC4=CC(=O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O[C@@]1([C@@H]2[C@](CCC(C2)=C(CO)C)(CCC1)C)C', 'Substituted "
               "hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@]12CC[C@@]3(C)[C@@]([H])(CC[C@@]33CCC(=O)O3)[C@]1([H])[C@@H](CC1=CC(=O)CC[C@]21C)SC(C)=O', "
               "'Substituted hexahydronaphthalene with substituents: C, S'), "
               "('O([C@@H]1C([C@]2([C@@]([C@@]3([C@]([C@]4(C([C@]5([C@@](CC4)(CCC(C5)(C)C)C(O)=O)[H])=CC3)C)(CC2)C)[H])(CC1)C)[H])(C)C)[C@@H]6O[C@@H]([C@@H](O)[C@H](O[C@H](OCC(O)=O)[C@H](O)C(O)=O)[C@H]6O)C(O)=O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('[H][C@@]12CC[C@]3(C)[C@]([H])(CC=C4[C@]5([H])[C@@H](C)C(=C)CC[C@@]5(CC[C@@]34C)C(O)=O)[C@@]1(C)C[C@@H](O)[C@H](O)[C@@]2(C)CO', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('C[C@]12CC[C@H]3[C@@H](CC(=C(Br)Br)C4=CC(=O)CC[C@]34C)[C@@H]1CC[C@@H]2O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('O=C(O[C@@H](C(O)(C)C)C[C@@H](O)[C@H]([C@@H]1[C@@]2([C@@](C3=C([C@@]4([C@H](C([C@@H](O)CC4)(C)C)CC3)C)CC2)(C)CC1)C)COC(=O)C)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C1OC[C@]2(C1=CC[C@H]3C(CC[C@@H]([C@]23C)OC(=O)[C@@H](NC(=O)C)C(C)C)(C)C)O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C1O[C@@H]2[C@@](O[C@@H](C)[C@@H]([C@H]2C)OC(=O)/C=C\\\\C(=O)CCCCCC)(C)[C@H]([C@H]1C)OC(=O)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('CC(=O)O[C@@H]1C[C@H]2C(C)(C)C(=O)C=C[C@]2(C)[C@H]2CC[C@]3(C)[C@@H](C(=O)C(O)=C3c3ccoc3)[C@]12C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C12=CC=C(C=C1CC[C@@]3([C@@]2(CC[C@]4([C@]3(C[C@H]([C@H]4O)O)[H])C)[H])[H])O', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('C[C@@H]1CC2(SCC=N2)[C@]2(O)O[C@@H]3C[C@@]4(C)[C@@H](C[C@@H]5O[C@]55[C@@H]4[C@H](O)C(=O)[C@]4(C)[C@H](CC[C@]54O)C4=CC(=O)OC4)C[C@H]3O[C@@H]2O1', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('C[C@](O)(CC[C@@H]1C(=C)CC[C@@H]2C(C)(C)CCC[C@@]12C)C=C', "
               "'Substituted hexahydronaphthalene with substituents: C'), "
               "('[H][C@]12CC[C@]3([H])[C@@]4(C)CCC(=O)C(C)(C)[C@]4([H])CC[C@@]3(C)[C@]1(C)CC[C@]1(O)CC[C@H](O[C@]21[H])C(C)(C)O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('O=C(OC[C@]1(O)[C@H]2[C@@H](C=3[C@@]4([C@H]([C@@]([C@H](O)CC4)(CO)C)CCC3C2)C)CC1)C', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('CC(C)C[C@@H]1CN2CCC3=CC(=C(C=C3[C@H]2C[C@H]1O)OC)O', "
               "'Substituted hexahydronaphthalene with substituents: C, O'), "
               "('[H][C@]1(O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)O[C@H]1[C@H](O)[C@@H](CO[C@@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O[C@]2([H])O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2O)O[C@@H](OC(=O)[C@]23CCC(C)(C)C[C@@]2([H])C2=CC[C@]4([H])[C@@]5(C)CC[C@H](O)[C@@](C)(C(=O)O[C@@H]6O[C@H](CO)[C@@H](O)[C@H](O)[C@H]6O)[C@]5([H])CC[C@@]4(C)[C@]2(C)C[C@H]3O)[C@@H]1O', "
               "'Substituted hexahydronaphthalene with substituents: C, O')]\n"
               'False negatives: '
               "[('[C@@]12([C@@H](OC([C@H](CC)C)=O)C[C@H](O)C=C1C=C[C@@H]([C@@H]2CC[C@H]3OC(C[C@@H](C3)O)=O)C)[H]', "
               "'No hexahydronaphthalene core structure found')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 895,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.8996990972918756}


"""
Classifies: CHEBI:24272 glucosaminyl group
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.MolStandardize import rdMolStandardize

def is_glucosaminyl_group(smiles: str):
    """
    Determines if a molecule is a glucosaminyl group.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glucosaminyl group, False otherwise
        str: Reason for classification
    """
    # Check for valid SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Look for key structural features:
    # 1. Must have N-acetyl group (-NC(=O)CH3)
    # 2. Must have pyranose ring structure
    # 3. Must have a free position where hemiacetal OH was removed (marked with *)
    
    # Check for N-acetyl group
    nacetyl_pattern = Chem.MolFromSmarts('[NH][C](=O)[CH3]')
    if not mol.HasSubstructMatch(nacetyl_pattern):
        return False, "Missing N-acetyl group"
        
    # Check for pyranose ring
    pyranose_pattern = Chem.MolFromSmarts('[C][O][C]1[C][C][C][C][O]1')
    if not mol.HasSubstructMatch(pyranose_pattern):
        return False, "Missing pyranose ring structure"
        
    # Check for * marking position of removed hemiacetal OH
    has_star = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == '*':
            has_star = True
            break
    if not has_star:
        return False, "Missing * indicating position of removed hemiacetal OH"
        
    # Additional check for connectivity - * should be connected to pyranose ring
    star_atom = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == '*':
            star_atom = atom
            break
            
    if star_atom:
        neighbor_atoms = [n for n in star_atom.GetNeighbors()]
        if len(neighbor_atoms) != 1:
            return False, "* must be connected to exactly one atom"
        neighbor = neighbor_atoms[0]
        if neighbor.GetSymbol() != 'C':
            return False, "* must be connected to carbon atom"
    
    return True, "Contains N-acetylglucosamine with removed hemiacetal OH"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24272',
                          'name': 'glucosaminyl group',
                          'definition': 'A glycosaminyl group obtained by '
                                        'removing the hydroxy group from the '
                                        'hemiacetal function of a glucosamine '
                                        'and, by extension, of a lower '
                                        'oligosaccharide having a glucosamine '
                                        'at the reducing end.',
                          'parents': ['CHEBI:24399']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 8,
    'num_false_positives': 100,
    'num_true_negatives': 12724,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.07407407407407407,
    'recall': 1.0,
    'f1': 0.13793103448275862,
    'accuracy': 0.9922069825436409}


"""
Classifies: CHEBI:28936 2-acyl-sn-glycero-3-phosphoethanolamine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_2_acyl_sn_glycero_3_phosphoethanolamine(smiles: str):
    """
    Determines if a molecule is a 2-acyl-sn-glycero-3-phosphoethanolamine.

    A 2-acyl-sn-glycero-3-phosphoethanolamine is a lysophosphatidylethanolamine
    obtained by selective hydrolysis of the 1-acyl substituent of any 1,2-diacyl-sn-glycero-3-phosphoethanolamine.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a 2-acyl-sn-glycero-3-phosphoethanolamine, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all phosphorus atoms
    phosphorus_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'P']

    if len(phosphorus_atoms) != 1:
        return False, "The molecule does not contain exactly one phosphorus atom"

    p_atom_idx = phosphorus_atoms[0]
    p_atom = mol.GetAtomWithIdx(p_atom_idx)

    # Check if the phosphorus atom has four neighbors
    if p_atom.GetDegree() != 4:
        return False, "The phosphorus atom does not have four neighbors"

    # Check if one neighbor is a doubly bonded oxygen (phosphate group)
    has_phosphate_group = False
    for neighbor_idx in p_atom.GetNeighbors():
        neighbor_atom = mol.GetAtomWithIdx(neighbor_idx)
        if neighbor_atom.GetSymbol() == 'O' and mol.GetBondBetweenAtoms(p_atom_idx, neighbor_idx).GetBondType() == Chem.BondType.DOUBLE:
            has_phosphate_group = True
            break

    if not has_phosphate_group:
        return False, "The molecule does not have a phosphate group"

    # Check if one neighbor is an ethanolamine group
    has_ethanolamine_group = False
    for neighbor_idx in p_atom.GetNeighbors():
        neighbor_atom = mol.GetAtomWithIdx(neighbor_idx)
        if neighbor_atom.GetSymbol() == 'O':
            # Follow the chain to find an ethanolamine group
            chain = Chem.FindAllPathsOfLengthN(mol, neighbor_idx, 3)
            if chain:
                chain_atoms = [mol.GetAtomWithIdx(atom_idx) for atom_idx in chain[0]]
                if all(atom.GetSymbol() in ['C', 'N'] for atom in chain_atoms):
                    has_ethanolamine_group = True
                    break

    if not has_ethanolamine_group:
        return False, "The molecule does not have an ethanolamine group"

    # Check if one neighbor is a glycerol group
    has_glycerol_group = False
    for neighbor_idx in p_atom.GetNeighbors():
        neighbor_atom = mol.GetAtomWithIdx(neighbor_idx)
        if neighbor_atom.GetSymbol() == 'O':
            # Follow the chain to find a glycerol group
            chain = Chem.FindAllPathsOfLengthN(mol, neighbor_idx, 3)
            if chain:
                chain_atoms = [mol.GetAtomWithIdx(atom_idx) for atom_idx in chain[0]]
                if all(atom.GetSymbol() in ['C', 'O'] for atom in chain_atoms):
                    has_glycerol_group = True
                    break

    if not has_glycerol_group:
        return False, "The molecule does not have a glycerol group"

    # Check if the remaining neighbor is an acyl group
    has_acyl_group = False
    for neighbor_idx in p_atom.GetNeighbors():
        neighbor_atom = mol.GetAtomWithIdx(neighbor_idx)
        if neighbor_atom.GetSymbol() == 'C':
            has_acyl_group = True
            break

    if not has_acyl_group:
        return False, "The molecule does not have an acyl group"

    return True, "The molecule is a 2-acyl-sn-glycero-3-phosphoethanolamine"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:28936',
                          'name': '2-acyl-sn-glycero-3-phosphoethanolamine',
                          'definition': 'A lysophosphatidylethanolamine '
                                        'obtained by selective hydrolysis of '
                                        'the 1-acyl substituent of any '
                                        '1,2-diacyl-sn-glycero-3-phosphoethanolamine.',
                          'parents': ['CHEBI:64574']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.GetAtomWithIdx(Mol, Atom)\n'
             'did not match C++ signature:\n'
             '    GetAtomWithIdx(RDKit::ROMol {lvalue} self, unsigned int idx)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:24613 homodetic cyclic peptide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors


def is_homodetic_cyclic_peptide(smiles: str):
    """
    Determines if a molecule is a homodetic cyclic peptide.
    A homodetic cyclic peptide has a ring consisting solely of amino acid residues 
    connected by peptide bonds.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a homodetic cyclic peptide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Generate the ring information
    rings = mol.GetRingInfo()
    if not rings.NumRings():
        return False, "No rings found"

    # Look for peptide bonds (-C(=O)-N-) in rings
    peptide_bonds = []
    for bond in mol.GetBonds():
        # Get the atoms connected by this bond
        atom1 = bond.GetBeginAtom()
        atom2 = bond.GetEndAtom()
        
        # Check if this is a C-N single bond
        if bond.GetBondType() == Chem.BondType.SINGLE and \
           atom1.GetSymbol() in ['C', 'N'] and atom2.GetSymbol() in ['C', 'N']:
            
            # For C atom, check if it has a double-bonded O
            c_atom = atom1 if atom1.GetSymbol() == 'C' else atom2
            n_atom = atom2 if atom1.GetSymbol() == 'C' else atom1
            
            has_carbonyl = False
            for neighbor in c_atom.GetNeighbors():
                if neighbor.GetSymbol() == 'O' and \
                   mol.GetBondBetweenAtoms(c_atom.GetIdx(), neighbor.GetIdx()).GetBondType() == Chem.BondType.DOUBLE:
                    has_carbonyl = True
                    break
                    
            if has_carbonyl:
                peptide_bonds.append((c_atom.GetIdx(), n_atom.GetIdx()))

    if not peptide_bonds:
        return False, "No peptide bonds found"

    # Check if peptide bonds form a cycle
    bond_atoms = set()
    for c, n in peptide_bonds:
        bond_atoms.add(c)
        bond_atoms.add(n)

    # Get all ring atoms
    ring_atoms = set()
    for ring in rings.AtomRings():
        ring_atoms.update(ring)

    # Check if peptide bonds are part of a ring
    peptide_ring = False
    for ring in rings.AtomRings():
        ring_set = set(ring)
        if any(c in ring_set and n in ring_set for c, n in peptide_bonds):
            peptide_ring = True
            break

    if not peptide_ring:
        return False, "No cyclic peptide structure found"

    # Additional checks for amino acid residues
    for ring in rings.AtomRings():
        ring_set = set(ring)
        peptide_bonds_in_ring = [(c,n) for c,n in peptide_bonds if c in ring_set and n in ring_set]
        if len(peptide_bonds_in_ring) >= 2:  # Need at least 2 peptide bonds for cyclic peptide
            return True, "Homodetic cyclic peptide structure found"

    return False, "Structure does not match homodetic cyclic peptide pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24613',
                          'name': 'homodetic cyclic peptide',
                          'definition': 'A homodetic cyclic peptide is a '
                                        'cyclic peptide in which the ring '
                                        'consists solely of amino-acid '
                                        'residues in peptide linkages.',
                          'parents': ['CHEBI:23449']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 15,
    'num_false_positives': 100,
    'num_true_negatives': 779,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.13043478260869565,
    'recall': 0.9375,
    'f1': 0.22900763358778628,
    'accuracy': 0.8871508379888268}


"""
Classifies: CHEBI:25497 neolignan
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors
from rdkit.Chem import rdDecomposition

def is_neolignan(smiles: str):
    """
    Determines if a molecule is a neolignan based on structural characteristics.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a neolignan, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for minimum size - neolignans should have at least 2 phenylpropanoid units
    num_atoms = mol.GetNumAtoms()
    if num_atoms < 18:  # Minimum size for 2 phenylpropanoid units
        return False, "Molecule too small to be a neolignan"

    # Look for aromatic rings
    aromatic_rings = []
    for ring in mol.GetRingInfo().AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings.append(ring)

    if len(aromatic_rings) < 2:
        return False, "Insufficient number of aromatic rings"

    # Check for typical neolignan features
    has_methoxy = False
    has_hydroxy = False
    has_propyl = False
    
    # Look for methoxy groups (-OCH3)
    pattern_methoxy = Chem.MolFromSmarts('cOC')
    if mol.HasSubstructMatch(pattern_methoxy):
        has_methoxy = True
        
    # Look for hydroxy groups (-OH)
    pattern_hydroxy = Chem.MolFromSmarts('cO[H]')
    if mol.HasSubstructMatch(pattern_hydroxy):
        has_hydroxy = True
        
    # Look for propyl chain connections
    pattern_propyl = Chem.MolFromSmarts('ccC[C@H]')
    if mol.HasSubstructMatch(pattern_propyl):
        has_propyl = True

    # Check for ether linkages between units (typical in neolignans)
    pattern_ether = Chem.MolFromSmarts('cOc')
    has_ether = mol.HasSubstructMatch(pattern_ether)

    # Check for beta-carbon coupling (should not be present in neolignans)
    pattern_beta = Chem.MolFromSmarts('ccCC(C)CC')
    beta_coupled = mol.HasSubstructMatch(pattern_beta)

    if beta_coupled:
        return False, "Contains beta-carbon coupling typical of lignans"

    # Evaluate overall structure
    if has_methoxy and has_hydroxy and has_propyl and has_ether:
        features = []
        if has_methoxy:
            features.append("methoxy groups")
        if has_hydroxy:
            features.append("hydroxy groups")
        if has_propyl:
            features.append("propyl chains")
        if has_ether:
            features.append("ether linkages")
            
        return True, f"Neolignan structure confirmed with: {', '.join(features)}"
    else:
        missing = []
        if not has_methoxy:
            missing.append("methoxy groups")
        if not has_hydroxy:
            missing.append("hydroxy groups")
        if not has_propyl:
            missing.append("propyl chains")
        if not has_ether:
            missing.append("ether linkages")
            
        return False, f"Missing key neolignan features: {', '.join(missing)}"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25497',
                          'name': 'neolignan',
                          'definition': 'A phenylpropanoid that includes plant '
                                        'products formed primarily from '
                                        'oxidative coupling of two '
                                        'para-propylphenol moities. Unlike '
                                        'lignan, the units are coupled at  '
                                        'positions other than their '
                                        'beta-carbon atoms.',
                          'parents': ['CHEBI:26004']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "cannot import name 'rdDecomposition' from 'rdkit.Chem' "
             '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:25479 naphthalenone
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_naphthalenone(smiles: str):
    """
    Determines if a molecule is a naphthalenone (naphthalene with at least one oxo substituent).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a naphthalenone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find naphthalene core
    naphthalene_pattern = Chem.MolFromSmarts('c1ccc2ccccc2c1')
    if not mol.HasSubstructMatch(naphthalene_pattern):
        return False, "No naphthalene core found"
    
    # Find ketone/oxo groups
    ketone_pattern = Chem.MolFromSmarts('C(=O)')
    ketone_matches = mol.GetSubstructMatches(ketone_pattern)
    
    if not ketone_matches:
        return False, "No oxo substituents found"

    # Check if any ketone is connected to naphthalene core
    naphthalene_atoms = set()
    for match in mol.GetSubstructMatches(naphthalene_pattern):
        naphthalene_atoms.update(match)

    ketone_connected = False
    for match in ketone_matches:
        carbon_idx = match[0]
        # Get neighbors of the ketone carbon
        neighbors = [n.GetIdx() for n in mol.GetAtomWithIdx(carbon_idx).GetNeighbors()]
        # Check if any neighbor is part of naphthalene core
        if any(n in naphthalene_atoms for n in neighbors):
            ketone_connected = True
            break

    if not ketone_connected:
        return False, "Oxo groups not connected to naphthalene core"

    num_oxo = len(ketone_matches)
    return True, f"Naphthalene with {num_oxo} oxo substituent(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25479',
                          'name': 'naphthalenone',
                          'definition': 'Any naphthalene that is carrying at '
                                        'least one oxo substituent.',
                          'parents': ['CHEBI:25477', 'CHEBI:3992']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 100,
    'num_true_negatives': 17523,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9942127659574468}


"""
Classifies: CHEBI:25901 pentose
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_pentose(smiles: str):
    """
    Determines if a molecule is a pentose (5-carbon monosaccharide).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a pentose, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Count carbons
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count != 5:
        return False, f"Contains {carbon_count} carbons, pentoses must have exactly 5"
        
    # Count oxygens 
    oxygen_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if oxygen_count < 4:
        return False, f"Contains only {oxygen_count} oxygens, pentoses require at least 4"

    # Check for aldehyde (C=O) or ketone (C(=O)C) group
    has_aldehyde = False
    has_ketone = False
    
    # Look for aldehyde pattern
    patt_ald = Chem.MolFromSmarts('[CH]=O')
    if mol.HasSubstructMatch(patt_ald):
        has_aldehyde = True
        
    # Look for ketone pattern 
    patt_ket = Chem.MolFromSmarts('CC(=O)C')
    if mol.HasSubstructMatch(patt_ket):
        has_ketone = True

    # Check for cyclic forms (furanose/pyranose)
    ring_info = mol.GetRingInfo()
    has_ring = ring_info.NumRings() > 0
    
    if has_ring:
        # Check ring sizes
        ring_sizes = [len(ring) for ring in ring_info.AtomRings()]
        if 5 in ring_sizes:
            return True, "Furanose form of pentose"
        elif 6 in ring_sizes:
            return True, "Pyranose form of pentose"
    elif has_aldehyde:
        return True, "Linear aldopentose"
    elif has_ketone:
        return True, "Linear ketopentose"
        
    return False, "Does not match pentose structural patterns"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25901',
                          'name': 'pentose',
                          'definition': 'A five-carbon monosaccharide which in '
                                        'its linear form contains either an '
                                        'aldehyde group at position 1 '
                                        '(aldopentose) or a ketone group at '
                                        'position 2 (ketopentose).',
                          'parents': ['CHEBI:35381']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 8,
    'num_false_positives': 100,
    'num_true_negatives': 54932,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.07407407407407407,
    'recall': 0.8888888888888888,
    'f1': 0.13675213675213674,
    'accuracy': 0.9981650042695446}


"""
Classifies: CHEBI:195615 dodecanediol
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_dodecanediol(smiles: str):
    """
    Determines if a molecule is a dodecanediol (a diol that is dodecane carrying two hydroxy groups at unspecified positions).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a dodecanediol, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains 12 carbon atoms and 2 oxygen atoms
    if Descriptors.HeavyAtomCount(mol) != 14:
        return False, "Molecule does not contain 12 carbon atoms and 2 oxygen atoms"

    # Check if the molecule contains exactly 2 hydroxyl groups
    hydroxyl_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O' and atom.GetHybridization() == Chem.HybridizationType.SP3)
    if hydroxyl_count != 2:
        return False, "Molecule does not contain exactly 2 hydroxyl groups"

    # Check if the molecule has a linear chain of 12 carbon atoms
    sssr = Chem.GetSymmSSSR(mol)
    ring_count = len(sssr)
    if ring_count > 0:
        return False, "Molecule contains rings"

    # Check if the hydroxy groups are attached to the carbon chain
    atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'C']
    carbon_chain = []
    for atom in atoms:
        neighbors = [neighbor.GetSymbol() for neighbor in atom.GetNeighbors()]
        if 'O' in neighbors:
            carbon_chain.append(True)
        else:
            carbon_chain.append(False)

    if sum(carbon_chain) != 2:
        return False, "Hydroxy groups are not attached to the carbon chain"

    return True, "Molecule is a dodecanediol"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:195615',
                          'name': 'dodecanediol',
                          'definition': 'A diol that is dodecane carrying two '
                                        'hydroxy groups at unspecified '
                                        'positions.',
                          'parents': [   'CHEBI:23824',
                                         'CHEBI:24026',
                                         'CHEBI:50584']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.02631578947368421 is too low.\n'
               "True positives: [('CCCC(O)CCCCCCCCO', 'Molecule is a "
               "dodecanediol')]\n"
               "False positives: [('OC(CC#CC#CC#CCO)C(O)=O', 'Molecule is a "
               "dodecanediol'), ('OC(CCCCCCCO)CC(O)=O', 'Molecule is a "
               "dodecanediol'), ('O=C(O)CNCP(=O)(O)O.NC(C)C', 'Molecule is a "
               "dodecanediol'), ('N[C@@H](CCC(O)=O)C(=O)OP(O)(O)=O', 'Molecule "
               "is a dodecanediol'), "
               "('O=C/C(=C/C=C/C)/[C@@H](O)[C@@H](O)/C=C\\\\C', 'Molecule is a "
               "dodecanediol'), "
               "('[NH3+][C@@H](CCC(=O)OP([O-])([O-])=O)C([O-])=O', 'Molecule "
               "is a dodecanediol'), ('COC(=O)C=C(C)OP(=O)(OC)OC', 'Molecule "
               "is a dodecanediol'), ('O[C@]([C@H](O)C(O)=O)(CC(O)=O)C(O)=O', "
               "'Molecule is a dodecanediol'), ('CC(C)CCCCCCOS(O)(=O)=O', "
               "'Molecule is a dodecanediol'), ('CS(=O)(=O)OCCCCOS(C)(=O)=O', "
               "'Molecule is a dodecanediol'), "
               "('O=C(O)C[C@H](O)C[C@H](O)CCCCC', 'Molecule is a "
               "dodecanediol'), ('CC(C)CCC[C@@H](C)COS([O-])(=O)=O', 'Molecule "
               "is a dodecanediol'), ('CC[C@@H](CO)NCCN[C@@H](CC)CO', "
               "'Molecule is a dodecanediol'), ('COP(=S)(OC)SCC(=O)N(C)C=O', "
               "'Molecule is a dodecanediol'), "
               "('[H]C(OP([O-])([O-])=O)=C(O)C(=O)CCSC', 'Molecule is a "
               "dodecanediol'), ('OCCCCCCC[C@@H](O)CC(O)=O', 'Molecule is a "
               "dodecanediol'), "
               "('C(C(=O)NP([O-])(=O)[O-])C[C@@H](C(=O)[O-])[NH3+]', 'Molecule "
               "is a dodecanediol'), "
               "('C(=O)([C@@H](N)CO)N[C@H](C(=O)O)[C@@H](C)O', 'Molecule is a "
               "dodecanediol'), ('[H]C(OP(O)(O)=O)=C(O)C(=O)CCSC', 'Molecule "
               "is a dodecanediol'), ('CC[C@H](CO)NCCN[C@H](CC)CO', 'Molecule "
               "is a dodecanediol'), ('CCOP(=S)(OCC)SCCSCC', 'Molecule is a "
               "dodecanediol'), ('OC(C(NC(=O)C(N)CO)C(O)=O)C', 'Molecule is a "
               "dodecanediol'), ('O=C(O)/C=C/C=C/CCC[C@H](O)CO', 'Molecule is "
               "a dodecanediol'), ('O(CCCCCCCCC)S([O-])(=O)=O', 'Molecule is a "
               "dodecanediol'), ('P(OC(=O)[C@@H](N)CCCCN)(O)(O)=O', 'Molecule "
               "is a dodecanediol'), ('ClC/C(=C\\\\CO)/CC[C@H](OC)C(=C)C', "
               "'Molecule is a dodecanediol'), "
               "('C[C@@H](O)CCCCC[C@@H](O)CC(O)=O', 'Molecule is a "
               "dodecanediol'), ('NC(CCC(=O)OP(O)(O)=O)C(O)=O', 'Molecule is a "
               "dodecanediol'), ('CC(C)CCC[C@@H](C)COS(O)(=O)=O', 'Molecule is "
               "a dodecanediol'), ('CCOP(=O)(OCC)SCCSCC', 'Molecule is a "
               "dodecanediol'), ('N[C@@H](CCC(=O)OP(O)(O)=O)C(O)=O', 'Molecule "
               "is a dodecanediol'), ('OC(CCCCC)CC(O)CC(O)=O', 'Molecule is a "
               "dodecanediol'), ('CC(C)CCCC(C)COS(O)(=O)=O', 'Molecule is a "
               "dodecanediol'), ('OC(C(O)C(O)=O)C#CCCCCC', 'Molecule is a "
               "dodecanediol'), ('OC(CCCCC/C=C/C(O)=O)CO', 'Molecule is a "
               "dodecanediol'), ('OC(=O)CN(CC(O)=O)CP(O)(O)=O', 'Molecule is a "
               "dodecanediol'), ('C(CCCC(=O)NO)CCC(=O)NO', 'Molecule is a "
               "dodecanediol'), ('CNC(=O)\\\\C=C(/C)OP(=O)(OC)OC', 'Molecule "
               "is a dodecanediol'), ('O=C(O)[C@@H](N(O)O)CCCCCSC', 'Molecule "
               "is a dodecanediol'), ('O=C(/C=C/C=C/C)C[C@@H](OC)CCO', "
               "'Molecule is a dodecanediol'), "
               "('N[C@H](CCC(=O)OP(O)(O)=O)C(O)=O', 'Molecule is a "
               "dodecanediol'), ('CCS(=O)(=O)CCSP(=O)(OC)OC', 'Molecule is a "
               "dodecanediol'), ('CCC(CO)NCCNC(CC)CO', 'Molecule is a "
               "dodecanediol'), ('C(C(=O)NP(O)(=O)O)C[C@@H](C(=O)O)N', "
               "'Molecule is a dodecanediol'), "
               "('C(C[NH3+])CCC[NH2+]CC([C@@H](CO)O)=O', 'Molecule is a "
               "dodecanediol'), ('CC(C)CCCC(C)COS([O-])(=O)=O', 'Molecule is a "
               "dodecanediol'), ('NC(CCC(O)=O)C(=O)OP(O)(O)=O', 'Molecule is a "
               "dodecanediol'), "
               "('[H]C([H])(C=O)[C@](C)([N+]([O-])=O)[C@@]([H])(OC)[C@]([H])(C)O', "
               "'Molecule is a dodecanediol'), "
               "('C[C@@H](O)[C@H]([NH3+])C(=O)N[C@@H](CO)C([O-])=O', 'Molecule "
               "is a dodecanediol'), ('O=C(O)CC/C=C/[C@H](O)CC[C@H](O)C', "
               "'Molecule is a dodecanediol'), ('CSCCCCCC(N(O)O)C(O)=O', "
               "'Molecule is a dodecanediol'), ('OC(C(O)C(OCC)=O)C(OCC)=O', "
               "'Molecule is a dodecanediol'), "
               "('O.OC(=O)CC(O)(CC(O)=O)C(O)=O', 'Molecule is a "
               "dodecanediol'), ('COC(=O)C(\\\\C)=C\\\\OP(=S)(OC)OC', "
               "'Molecule is a dodecanediol'), "
               "('OC(C(O)C(O)=O)(CC(O)=O)C(O)=O', 'Molecule is a "
               "dodecanediol'), ('OC(C(N)C(=O)NC(CO)C(O)=O)C', 'Molecule is a "
               "dodecanediol'), "
               "('[NH3+][C@@H](CCC([O-])=O)C(=O)OP([O-])([O-])=O', 'Molecule "
               "is a dodecanediol'), ('O=C(O)C=CNC(=O)[C@H](O)C(O)(C)C', "
               "'Molecule is a dodecanediol'), "
               "('ClC/C(=C\\\\COC)/CC[C@H](O)C(=C)C', 'Molecule is a "
               "dodecanediol'), ('CCS(=O)CC(C)SP(=O)(OC)OC', 'Molecule is a "
               "dodecanediol'), ('C[C@@H](O)[C@H](N)C(=O)N[C@@H](CO)C(O)=O', "
               "'Molecule is a dodecanediol'), ('CC(C)CCCCCCOS([O-])(=O)=O', "
               "'Molecule is a dodecanediol'), ('CSCCCCCC(N(O)O)C([O-])=O', "
               "'Molecule is a dodecanediol'), "
               "('[H+].[H+].NCCS.OC(C(O)C([O-])=O)C([O-])=O', 'Molecule is a "
               "dodecanediol'), ('N[C@H](CCC(O)=O)C(=O)OP(O)(O)=O', 'Molecule "
               "is a dodecanediol'), ('O(C(OC)CCCCCCCCC)C', 'Molecule is a "
               "dodecanediol'), ('O(CCC(C)C)C(OCCC(C)C)C', 'Molecule is a "
               "dodecanediol'), "
               "('C[C@@H](O)[C@H](NC(=O)[C@@H]([NH3+])CO)C([O-])=O', 'Molecule "
               "is a dodecanediol'), ('O=C([O-])[C@@H](N(O)O)CCCCCSC', "
               "'Molecule is a dodecanediol'), "
               "('CC(C)CCC[C@H](C)COS([O-])(=O)=O', 'Molecule is a "
               "dodecanediol'), ('O(C(C=C(CCC=C(C)C)C)OC)C', 'Molecule is a "
               "dodecanediol'), ('O=C(O)\\\\C=C/C=C/[C@H](O)CC[C@H](O)C', "
               "'Molecule is a dodecanediol'), "
               "('CC(C)CCC[C@H](C)COS(O)(=O)=O', 'Molecule is a "
               "dodecanediol'), ('CCC(C)(O)C#CC#CC(C)(O)CC', 'Molecule is a "
               "dodecanediol')]\n"
               'False negatives: []',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 21,
    'num_true_negatives': 183903,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.045454545454545456,
    'recall': 1.0,
    'f1': 0.08695652173913045,
    'accuracy': 0.9998858230256898}


"""
Classifies: CHEBI:138397 octahydronaphthalenes
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_octahydronaphthalenes(smiles: str):
    """
    Determines if a molecule is an octahydronaphthalene or derivative.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an octahydronaphthalene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Create SMARTS patterns for decalin core with different saturation patterns
    # Allow for different arrangements of single/double bonds
    patterns = [
        # Basic decalin cores with varying degrees of unsaturation
        "C1CCC2CCCCC2C1",  # Fully saturated
        "C1CC=C2CCCCC2C1", # One double bond
        "C1CCC2C=CCCC2C1", # One double bond in different position
        "C1C=CC2CCCCC2C1", # One double bond in different position
        # Additional patterns with different bond arrangements
        "C1CCC2CCCC=C2C1",
        "C1CC=C2CC=CCC2C1",
        "[#6]1~[#6]~[#6]~[#6]2~[#6]~[#6]~[#6]~[#6]~[#6]2~[#6]1" # Generic carbon backbone
    ]

    found_match = False
    matched_pattern = None
    for pattern in patterns:
        patt = Chem.MolFromSmarts(pattern)
        if patt and mol.HasSubstructMatch(patt):
            found_match = True
            matched_pattern = patt
            break

    if not found_match:
        return False, "Does not match octahydronaphthalene core structure"

    # Get all atoms in the matching substructure
    matches = mol.GetSubstructMatches(matched_pattern)
    if not matches:
        return False, "Does not match octahydronaphthalene core structure"

    # Check the first match (we only need one valid match)
    match_atoms = set(matches[0])
    
    # Count double bonds in the matched core
    double_bonds_in_core = 0
    for bond in mol.GetBonds():
        if bond.GetBeginAtomIdx() in match_atoms and bond.GetEndAtomIdx() in match_atoms:
            if bond.GetBondType() == Chem.rdchem.BondType.DOUBLE:
                double_bonds_in_core += 1

    # Allow up to 2 double bonds in the core to account for various substitution patterns
    if double_bonds_in_core > 2:
        return False, "Too many double bonds in core structure"

    # Check that matched atoms are carbons (but allow for substitution)
    core_carbons = 0
    for atom_idx in match_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        if atom.GetSymbol() == 'C':
            core_carbons += 1

    if core_carbons < 8:  # Need at least 8 carbons in the core
        return False, "Insufficient carbon atoms in core structure"

    return True, "Matches octahydronaphthalene core structure with valid substitution pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:138397',
                          'name': 'octahydronaphthalenes',
                          'definition': 'Any carbobycyclic compound that is an '
                                        'octahydronaphthalene or a compound '
                                        'obtained from an octahydronaphthalene '
                                        'by formal substitution of one or more '
                                        'hydrogens.',
                          'parents': ['CHEBI:36785']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.13675213675213674 is too low.\n'
               'True positives: '
               "[('[H]C1(CCOC1=O)CC[C@@]1(C)[C@H](C)CC[C@@]2(C)C(=CCC[C@]12[H])C(O)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C=1CC[C@@H]([C@@]2(C1CC[C@H](C2)C(C)=C)C)C', 'Matches "
               'octahydronaphthalene core structure with valid substitution '
               "pattern'), "
               "('C[C@@H]1[C@@H](C[C@@H](O)C2=CC[C@H](C[C@]12C)C(C)=C)OC(C)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@]12C=C(CC[C@]1(C)CCCC2=C)C(C)C', 'Matches "
               'octahydronaphthalene core structure with valid substitution '
               "pattern'), ('C12=CCC[C@H]([C@]1(C[C@@H](CC2)C(=C)C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), ('C[C@H]1CCC[C@@]2(C)CCC=CC12', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC1=C(CC[C@H]2C(=C)CC[C@H]3C(C)(C)[C@@H](O)CC[C@]23C)[C@@]2(C)CCC(=O)C(C)(C)[C@@H]2CC1=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC(C)=CCCC(=C)[C@H]1CC[C@@]2(C)CCC=C(C)[C@@H]2C1', 'Matches "
               'octahydronaphthalene core structure with valid substitution '
               "pattern')]\n"
               'False positives: '
               "[('[H][C@@]12CC[C@@]3([H])C(C)(C)CCC[C@]3(C)[C@@]11CC[C@H](C1)C(C)=C2', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]12CC(C)(C)CC[C@@]1(CC[C@]1(C)C2=CC[C@]2([H])[C@@]3(C)CC[C@H](O)[C@@](C)(C(O)=O)[C@]3([H])CC[C@@]12C)C(O)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]12CC[C@](C)(CC1=CC[C@@]1([H])[C@@](C)(CO)CCC[C@]21C)C=C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C[C@@H]1O[C@@H](OC[C@H]2O[C@@H](O[C@@H]3[C@@H](O)[C@@H](O)CO[C@H]3O[C@H]3CC[C@@]4(C)[C@@H](CC[C@]5(C)[C@@H]4CC=C4[C@@H]6CC(C)(C)CC[C@@]6(CC[C@@]54C)C(O)=O)C3(C)C)[C@H](O)[C@@H](O)[C@@H]2O)[C@H](O)[C@H](O)[C@H]1O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C2=C([C@]3(CCC(C([C@@H]3C1)(C)C)=O)C)CC[C@]4([C@]2(CC[C@@H]4[C@H](C(=O)O)CCC(=C)C(C)C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O1[C@H]2[C@]3([C@]4([C@@]([C@](C[C@H]4O)([C@@H](C5OC(=O)C(=C(C5)C)C)C)[H])([C@@H](O)C[C@@]3([C@@]6([C@@](O)([C@@H]12)CC=CC6=O)C)[H])C)[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C1[C@@]2([C@]3(CC[C@]4([C@]([C@@]3(CC[C@@]2(C[C@@H](C1)O[C@@H]5O[C@@H]([C@H]([C@@H]([C@H]5O)O)O)C([O-])=O)[H])[H])(CC[C@@]4([C@@H](CCC([O-])=O)C)[H])[H])C)[H])C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC(C)=CCC[C@](C)(O[C@@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1O)[C@H]1CC[C@]2(C)[C@@H]1[C@H](O)C[C@@H]1[C@@]3(C)CC[C@H](O[C@@H]4O[C@H](CO)[C@@H](O)[C@H](O)[C@H]4O)C(C)(C)[C@@H]3CC[C@@]21C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC1(C)O[C@@H]2C[C@H]3[C@@H]4C[C@H](F)C5=CC(=O)C=C[C@]5(C)[C@H]4[C@@H](O)C[C@]3(C)[C@@]2(O1)C(=O)CO', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C(O)C1=C(O)C2=C(O[C@@]3(CC[C@@H]4[C@@]([C@H]3C2)(CC[C@@H](C4(C)C)OC(=O)C)C)C)C=C1C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@@H]1[C@](C2[C@@]([C@@]3([C@]([C@]4(C([C@@]5([C@@](CC4)(CC[C@@](C5)(C)C(O)=O)C(O)=O)[H])=CC3)C)(CC2)C)[H])(CC1)C)(CO)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC(=C)[C@H]1CC2=C(C)C(=O)CC[C@@]2(C)CC1O', 'Matches "
               'octahydronaphthalene core structure with valid substitution '
               "pattern'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)O[C@H]1CC[C@]2(C)[C@H]3CC[C@]4(C)[C@H](CC[C@H]4[C@@H]3CC=C2C1)[C@H](C)CC[C@H](O)C(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('OC1C2(C3(C(C4(C(CC3)C(C(O[C@@H]5O[C@@H]([C@@H](O)[C@H](O)[C@H]5O[C@@H]6O[C@@H]([C@H](O)[C@H](O[C@@H]7OC[C@H](O)[C@H](O)[C@H]7O[C@@H]8O[C@@H]([C@@H](O)[C@H](O)[C@H]8O)CO)[C@H]6O)CO)C(O)=O)CC4)(C)C)C)CC=C2C9C(C1O)(C(OC(=O)/C(/C)=C\\\\C)CC(C9)(C)C)CO)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@]12CC=C3[C@]4([H])[C@@H](C)[C@H](C)CC[C@@]4(CC[C@@]3(C)[C@]1(C)CC[C@@]1(O)C(C)(C)[C@@H](O)[C@H](O)C[C@]21C)C(O)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[C@@]12([C@]3([C@](C[C@@]1([C@]4([H])CC[C@H]([C@]4(C3)[H])C)C=O)(C=C2C(C)C)[H])CO[C@]5([C@H]([C@@H]([C@@H]([C@H](O5)C)O)O)O)[H])C(O)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('FC1=C2C[C@]3(C[C@@]4([C@](O)(C(=O)C3=C(O)C2=C(O)C(NC(=O)CN5CCCC5)=C1)C(O)=C(C(=O)[C@H]4N(C)C)C(=O)N)[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@@H]1C([C@H]2[C@](C3=C([C@]4([C@]([C@@H]([C@@H]([C@H](O)CC=C(C)C)CO)CC4)(C)CC3)C)CC2)(C)CC1)(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C2=C([C@]3(CCC(C([C@@H]3C1)(C)C)=O)C)CC[C@]4([C@]2(CC[C@@H]4[C@H](CO)CC/C=C(/CO)\\\\C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CO[C@@H]1[C@@H](O)[C@H](C)O[C@@H](O[C@H]2CC[C@@]3(C=O)[C@H](CC[C@@H]4[C@@H]3[C@@H]3O[C@@H]3[C@]3(C)[C@H](CC[C@]43O)C3=CC(=O)OC3)C2)[C@H]1OC(C)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C[C@]12CC[C@H]3[C@@H](CCC4=C[C@@H](O)CC[C@]34C)[C@@H]1CC[C@@H]2O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]12CCC(=C)[C@@H](CC\\\\C(C)=C\\\\COP([O-])(=O)OP([O-])([O-])=O)[C@@]1(C)CCCC2(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C1[C@@]2([C@]3(CC[C@]4([C@]([C@@]3(CC[C@]2(CC(C1)=O)[H])[H])(CC[C@@H]4O[C@H]5[C@@H]([C@H]([C@@H]([C@H](O5)C(=O)O)O)O)O)[H])C)[H])C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C(OC)C1=CC(O)=C2O[C@@]3(CC2=C1)[C@]4([C@@H](C(CCC4)(C)C)CC[C@@H]3C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C[C@H](C[C@H](O)[C@H]1OC1(C)C)C1=C2C[C@H](O)[C@H]3[C@@]4(C)CCC(=O)C(C)(C)[C@@H]4CC[C@]3(C)[C@@]2(C)CC1', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('OC1C(C2(C(C3C(C4C(=CC3)C(C(OC5OC(C(O)C(O)C5O)COC6OC(C(O)C(O)C6O)CO)CC4)(C)C)(C(=O)C2)C)(C1)C)C)C(O)(C(O)CCC(O)(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('OCC12C(C3(C(C4(C(C5(C(CC4)C(C(=O)CC5)(C)C)C)CC3)C)=CC1)C)CC(CC2)(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]12CCC3CC(=O)CC[C@]3(C)[C@@]1([H])C(=O)C[C@@]1(C)[C@@]2([H])CC[C@]1(O)C(=O)CO', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@](O)(C(=O)[C@H]2C(=C([C@]1(C(=O)OC)[C@@]3([C@@H]2[C@@]45C(=O)O[C@@H](C3)[C@@H]4C([C@@H](OC(=O)C)CC5)(C)C)C)C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@@H](O[C@@H]2C(OC([C@H]2O)=O)(C)C)CC[C@]3(C1CC[C@@H]4[C@@]3(C=5NC=6C=CC=CC6C5C4)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C(C2C3(C4(C3)C(C5(C(CC4)(C(CC5)C(CC(O)/C=C(/C)\\\\C(O)=O)C)C)C)CC2)CC1)(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C2=C[C@@](C=C)(CC[C@@H]2[C@@]3(O)[C@H](O)CCC(C3=C1)(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C(OC[C@]1(O)[C@H]2C[C@@]3([C@@]4([C@H]([C@]5([C@H](OC(C)(C)OC5)CC4)C)CC[C@H]3C2)C)CC1)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@@]2(O[C@@H]2[C@]3(CCCC([C@@H]3C1)(C)C)C)C(=O)O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C2=C([C@@]3(C(=O)C[C@@H]([C@]3([C@@H]1OC(=O)C)C)C(=O)C)C)[C@@H](O)C[C@@H]4[C@@]2(CC[C@@H](C4(C)C)O)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@H](O)C2=C(CC[C@@]([C@@H]2O)(C=C)C)[C@@]3([C@@H]1C(CC[C@H]3O)(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[C@@]12([C@](CC[C@@H]([C@]13OC=4[C@](C3)(C(=C(C(C4C)=O)C(=O)[O-])C)C)C)([H])C(C(CC2)=O)(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O1[C@@]2([C@]([C@]3([C@@]([C@@]4([C@@](CC3)(CC5=C4NC6=C5C=7C[C@@]8([C@](C(OC8(C)C)(C)C)(C(=O)C7C=C6)[H])[H])[H])C)(CC2)C)[H])([C@H](O)C[C@]1(C(O)(C)C)[H])C)[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C1[C@@]2([C@@]([C@@]3([C@](C[C@H](OS([O-])(=O)=O)CC3)(C1)[H])C)(CC[C@@]4([C@H](CC[C@@]24[H])O)C)[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]1(CC[C@@]2([H])C3=CC[C@]4([H])[C@](C)(CC[C@H](O)[C@@]4(C)C(O)=O)[C@@]3([H])CC[C@]12C)[C@H](C)CCCC(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@]2(C(=C)[C@@](C[C@H]3[C@]2(CC[C@H]([C@@]3(CCC(=O)OC)C)C(O)(C)C)C)(C)C([C@]1(O)C)=O)C(=O)OC', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[C@@]12([C@]3([C@](CC[C@@]1([C@@]4(C(C[C@H](CC4)O)=CC2)C)[H])([C@](CC3)([C@H](C)CC[C@@H](C(C)C)OS([O-])(=O)=O)[H])C)[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@]2([C@]([C@]3([C@@]([C@H](CC3)C(=O)CO)(C1)CO)[H])(CC[C@]4([C@@]2(CC[C@@H](O)C4)C)[H])[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@]12[C@@]([C@H]3[C@H](O)C[C@@]4([C@@H]([C@@H]([C@@H]5[C@]([C@@H](C(C)C)C)(C)C5)C)CC[C@H]4[C@@H]3C[C@H]1O)C)(CC[C@@H](C2)O)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C(C1=CC[C@@H]2[C@@]1([C@H](O)C[C@@H]3[C@@]4(C(=C[C@H](O)[C@H](C4)O)CC=C23)C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@@]12[C@]([C@]3([C@](CC1)([C@@H](O)CC3)C)[H])(CCC=4[C@]2(C)C=CC(=O)C4)[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O1[C@@]([C@H]2C[C@H]3O[C@@H]1[C@@]42[C@@H]3[C@]5(O)[C@@H](O)C[C@H]6C[C@@H](O)CC[C@@]6([C@H]5CC4)C)(CCC(=C)C(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C(=C[C@H]2[C@@H](C(C)C)CC[C@]([C@H]2C1)(O)C)CO', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C\\\\1C[C@]2([C@]3([C@@]([C@@]4(C(=CC3)C[C@@H](O)CC4)C)(CC[C@@]2(/C1=C(\\\\C)/C(=O)CC[C@H](CO)C)C)[H])[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C(C2C(C3C(C4(C(C5C(CC4)(CCC5C(CO)=C)C)=CC3)C)(CC2)C)(C)C=C1)(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1OC[C@@H]2[C@]1([C@@H]3[C@@H](C=C2)C[C@@H](COC(=O)C)CC3)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@]12CC[C@@]3([H])[C@]4([H])CC[C@H](O)[C@@]4(C)CC[C@]3([H])[C@@]1(C)CC[C@H](O)C2', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]1(O[C@@H]2C[C@]3([H])[C@H](O)C[C@]4([H])[C@]([H])(CC[C@@]5(C)[C@@]4([H])C[C@]4([H])O[C@]6(CC[C@@H](C)CO6)[C@@H](C)[C@]54[H])[C@@]3(C)C[C@H]2O)O[C@H](CO)[C@H](O[C@]2([H])O[C@H](CO)[C@@H](O)[C@H](O[C@]3([H])O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)[C@H]2O[C@]2([H])O[C@H](CO)[C@@H](O)[C@H](O[C@]3([H])O[C@H](CO)[C@@H](O)[C@H](O)[C@H]3O)[C@H]2O)[C@H](O)[C@H]1O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C(O)=C2[C@@]([C@H](OC(=O)C)C[C@@H]3[C@]2(C3)[C@@]4(C1=C[C@@](C=C)(CC4)C)O)(COC(=O)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C1[C@]2([C@]3([C@@]([C@@]4(C(=CC(CC4)=O)[C@H](C3)C)C)(CC[C@@]2([C@@](C1)(O)C(=O)C)C)[H])[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@@H](O)[C@H]([C@@]2(CC3C(C2([C@@H]1C)CC3)=C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C[C@H]1[C@@H]2[C@H]3CC[C@@H]4[C@@]5(C)CC[C@H](O)C(C)(C)[C@@H]5CC[C@@]4(C)[C@]3(C)CC[C@@]2(C)CCC1=C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@]1([C@@]2([C@]([C@]3([C@@]([C@@]4([C@](CC3)(C[C@H](O)C=C4)[H])C)(CC2)[H])[H])(CC1)[H])C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[C@]12(CC[C@H](CC2C[C@@H](C3C4[C@](CCC13)(C(CC4)[C@@H](C(CCC(O)=O)O)O)C)O)O)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC(=O)OCC(=O)[C@H]1CC[C@@H]2[C@@]1(CC(=O)[C@H]3[C@H]2CC[C@@H]4[C@@]3(CC[C@H](C4)O)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C(O)=C2[C@@]([C@H](O)C[C@@H]3[C@]2(C3)[C@@]4(C1=C[C@@](C=C)(CC4)C)O)(COC(=O)C(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@]([C@@]1([C@@]2([C@]([C@]3([C@@]([C@@]4([C@@](CC3)(CC(=O)CC4)[H])C)(CC2)[H])[H])(CC1=O)[H])C)[H])(CCCC(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[C@@]12(C([C@@]3(CC(CC[C@@]3([C@H](C1)O)C([O-])=O)(C)C)[H])=CC[C@]4([C@]2(CC[C@@]5([C@@]4(CC[C@@H](C5(C)C)O)C)[H])C)[H])C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C[C@H](CCC[C@H](C)C(=O)SCCNC(=O)CCNC(=O)C(O)C(C)(C)COP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1OP(O)(O)=O)n1cnc2c(N)ncnc12)[C@H]1CC[C@H]2[C@@H]3[C@H](O)C[C@@H]4C[C@H](O)CC[C@]4(C)[C@H]3CC[C@]12C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])[C@H](O)C[C@]4([H])C[C@@H](CC[C@]4(C)[C@@]3([H])CC[C@]12C)OS(O)(=O)=O)[C@H](C)CCC(=O)NCCS(O)(=O)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC(C)=CCC[C@](C)(O[C@@H]1O[C@H](CO[C@@H]2OC[C@H](O)[C@H](O)[C@H]2O)[C@@H](O)[C@H](O)[C@H]1O)[C@H]1CC[C@]2(C)[C@@H]1[C@H](O)C[C@@H]1[C@@]3(C)CC[C@H](O)C(C)(C)[C@@H]3CC[C@@]21C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@]12O[C@]11CC[C@]3([H])C(C)(C)CCC[C@@]3(C)[C@]1([H])[C@@]1([H])O[C@]11OC(=O)C(CO)=C21', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C[C@@]1(CC[C@@]2(O)C(=C1)C(=O)C1=C3[C@@](C)(CC[C@@H](O)[C@]23C)CO1)C=C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@@H]1[C@@]([C@]2([C@@]([C@@]3([C@]([C@]4(C([C@]5([C@@](CC4)(CCC(C5)(C)C)C(O)=O)[H])=CC3)C)(CC2)C)[H])(C[C@@H]1O)C)[H])(C)C(O)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C12=C3C(=CC(=C2C(O[C@@H]([C@@H]1OC(C)=O)C)=O)O)O[C@@]4([C@]([C@]5(C=CC(C([C@@]5(C[C@@H]4O)[H])(C)C)=O)C)(C3)[H])C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@]1(COC(=O)C1)[C@@]1([H])CC[C@]2(O)[C@]3([H])CC[C@]4([H])C[C@H](CC[C@]4(C)[C@@]3([H])CC[C@]12C)O[C@H]1C[C@H](O)[C@H](O[C@H]2C[C@H](O)[C@H](O[C@H]3C[C@H](O)[C@H](O)[C@@H](C)O3)[C@@H](C)O2)[C@@H](C)O1', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1[C@H]2[C@H](C=C(C1)C(=O)O)[C@@H](C(C)C)CC[C@@H]2C(=O)O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O(CC1=C(O)C2=C(C[C@H]3[C@@]2(CC[C@@H]4[C@@]3(CCCC4(C)C)C)C)C(=C1)O)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1O[C@H](C)[C@@]2([C@@]13C(=C)[C@](C[C@@H]4[C@@]3(CC[C@H]5[C@]4([C@H](O)[C@H]6C(=O)O[C@]5(C)C6)C)C)(C)C2=O)O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O1CC2(C3(C(C4(C(CC3)C(C(O)CC4)(C)C)C)CCC2C(C(OC(=O)C)CC5OC(=O)C(C5)C)C)C)CC1=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC(=O)[C@H]1CC[C@H]2[C@@H]3CC[C@@H]4CC(=O)CC[C@]4(C)[C@H]3[C@H](O)C[C@]12C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C2=C(OC(=C1)C)O[C@]3(CC[C@H]4[C@]([C@@H]3C2)(CC[C@@H]5[C@@]4(CCC(C5(C)C)=O)C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]1(CC[C@@]2([H])[C@]3([H])CCC4=CC(=O)CC[C@]4(C)[C@@]3([H])CC[C@]12C)[C@H](C)CC[C@@H](C)C(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O[C@H]1[C@]2([C@]3([C@@]([C@](CC3)([C@@H](CC(O)=O)C)[H])(CC[C@@]2([C@@]4([C@]([C@@H]1O)(C[C@H](O)CC4)[H])C)[H])C)[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C=C2[C@@]3([C@H](C(C(=O)CC3)(C)C)C[C@@H]4[C@]2(O4)[C@]5([C@]1([C@@H](C(O)(CC(=O)CC(C(=O)O)C)C)CC5=O)C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O([C@]1([C@]2([C@](CC[C@H]1OC(=O)C(O)(C(OC(=O)C)C)C)(CC(=O)C(C2)=C(C)C)C)[H])C)C(=O)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O1C2C(C3(C(C4C(CC3)C5(C(=CC4)CC(O)CC5O[C@@H]6OC[C@H](OC(=O)C(O)C(CC)C)[C@H](O)[C@H]6O[C@@H]7O[C@H]([C@H](O)[C@@H](O)[C@H]7O)C)C)C2)C)C(C18OCC(CC8)=C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O1[C@@]2([C@@]([C@@]3([C@]([C@]4([C@](CC3)([C@@]5(C(=CC4)C[C@@H](O[C@@H]6O[C@@H]([C@@H](O)[C@H](O)[C@H]6O[C@@H]7O[C@H]([C@H](O)[C@@H](O)[C@H]7O)C)CO)CC5)C)[H])[H])(C2)[H])C)([C@@H]([C@]18OC[C@@H](CC8)C)C)[H])[H]', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O1C2C(C3(C(C4C(CC3)C5(C(=CC4)CC(OC6OC(C(O)C(O)C6O)COC7OC(C(O)C(O)C7OC8OC(C(O)C(O)C8O)C)CO)CC5)C)C2)C)C(C1(O)CCC(COC9OC(C(O)C(O)C9O)CO)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C(=C[C@H]2[C@H](C(=C)CO)CC[C@]([C@H]2C1)(O)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C[C@]1(CC[C@H]2C(CC[C@@H]3[C@]2(C)CCC[C@@]3(C)C(O)=O)=C1)C=C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C1[C@@]2([C@]3(CC[C@]4([C@]([C@@]3(CC[C@@]2(C[C@@H](C1)OS([O-])(=O)=O)[H])[H])(CC[C@@]4([C@H](C)CCC([O-])=O)[H])[H])C)[H])C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@]1(O)C[C@@]2(C)[C@@]([H])(CC[C@]2(C)O)[C@]2([H])CCC3=CC(=O)CC[C@]3(C)[C@@]12F', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C2=C(C3=C(NC4=C3C[C@@H]5CC[C@@]6([C@@]([C@@]45C)(CC[C@H]7[C@@]68O[C@@H]8[C@@H]9O[C@H](C%10OC%10(C)C)OC([C@H]9O7)(C)C)C)O)C=C2)C[C@@H]%11[C@@H]1C(OC%11(C)C)(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O(C1C2(C(C3(C(C(C(O)CC3)(C)C)C1)C)CCC4(C2=CC(OC4C=5C=COC5)=O)C)C)C(=O)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CC(=O)C=C[C@@H]1[C@]2(CCCC(C2CC[C@@]1(C)O)(C)C)C', 'Matches "
               'octahydronaphthalene core structure with valid substitution '
               "pattern'), "
               "('O=C1C2=C(C(=O)C[C@@H]3[C@@]2(CCC(C3(C)C)O)C)[C@@]4([C@H](O)C[C@@H]([C@]4([C@@H]1O)C)[C@@H](CC(=O)C[C@@H](C(=O)O)C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@@]12CC=C3C[C@H](CC[C@]3(C)[C@@]1([H])CC[C@]1(C)C(=CC[C@@]21[H])c1cccnc1)OC(C)=O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('CCN1C[C@]2(C)CC[C@H](O)[C@]34C1[C@H](C[C@H]23)[C@@]12C[C@H]([C@@H](O)C[C@@H]41)C(=C)[C@H]2O', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('C1[C@H](O)C([C@@]2(CC[C@@]3([C@](CC=C4[C@]3(CC[C@@]5([C@]4(C[C@@](C[C@@H]5O)(C(O)=O)C)[H])C)C)([C@]2(C1)C)[H])C)[H])(C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('[H][C@]1(O[C@@H](O[C@H](CO)[C@]2([H])O[C@@H](OCC(=C)C3CC[C@]4(C)C[C@@H](CC(C)=C4C3)O[C@@H]3O[C@]([H])([C@H](O)[C@H]3O)[C@@H](CO)O[C@@H]3O[C@@]([H])([C@H](O)CO)[C@H](O)[C@H]3O)[C@H](O)[C@H]2O)[C@H](O)[C@H]1O)[C@H](O)CO', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C(O)C1=C[C@H]2[C@@H](C(=C)CC[C@@H]2[C@H](C(=O)O)C)CC1', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('OC1CC2C(C3C(C4C(C(CC4)C(CCC(CC)C(C)=C)C)(CC3)C)=CC2)(CC1)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C(O)/C(=C/CC[C@H]([C@@H]1[C@@]2([C@@]([C@H]3[C@@]([C@@H](C(=C(C)C)CC3)CCCO)(C)CC2)(C)CC1)C)C)/C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern'), "
               "('O=C1C=C[C@@]2([C@H]3[C@]4([C@@]56[C@@H](C(=O)OC5)[C@@H](O)[C@@](C4)(C)C([C@@]6(C3)C)=O)CC[C@H]2C1(C)C)C', "
               "'Matches octahydronaphthalene core structure with valid "
               "substitution pattern')]\n"
               'False negatives: '
               "[('[C@H]1(COP([O-])([O-])=O)C(=CC[C@@]2([C@]1(C)CCCC2(C)C)[H])C', "
               "'Does not match octahydronaphthalene core structure')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 9,
    'num_false_positives': 100,
    'num_true_negatives': 795,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.08256880733944955,
    'recall': 1.0,
    'f1': 0.15254237288135594,
    'accuracy': 0.8893805309734514}


"""
Classifies: CHEBI:24269 glucosamine phosphate
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_glucosamine_phosphate(smiles: str):
    """
    Determines if a molecule is a glucosamine phosphate (a hexosamine phosphate having glucosamine as the amino sugar component).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a glucosamine phosphate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if it's a hexosamine phosphate
    if not Descriptors.NumHeteroatoms(mol) == 3:
        return False, "Not a hexosamine phosphate (incorrect number of heteroatoms)"

    if not Descriptors.NumHeterocycles(mol) == 2:
        return False, "Not a hexosamine phosphate (incorrect number of heterocycles)"

    # Check if it's glucosamine
    ring_atoms = mol.GetRingInfo().AtomRings()[0]
    ring_atoms = [mol.GetAtomWithIdx(idx) for idx in ring_atoms]
    ring_atoms = [atom for atom in ring_atoms if atom.GetSymbol() != 'O']

    if len(ring_atoms) != 6:
        return False, "Not a glucosamine phosphate (incorrect ring size)"

    if not all(atom.GetSymbol() in ['C', 'N'] for atom in ring_atoms):
        return False, "Not a glucosamine phosphate (incorrect ring atom types)"

    if not any(atom.GetSymbol() == 'N' for atom in ring_atoms):
        return False, "Not a glucosamine phosphate (no nitrogen in ring)"

    return True, "Glucosamine phosphate"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24269',
                          'name': 'glucosamine phosphate',
                          'definition': 'A hexosamine phosphate having '
                                        'glucosamine as the amino sugar '
                                        'component.',
                          'parents': ['CHEBI:24584']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.Descriptors' has no attribute "
             "'NumHeterocycles'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0,
    'f1': 0,
    'accuracy': None}


"""
Classifies: CHEBI:22487 alpha-aminoacyl group
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_alpha_aminoacyl_group(smiles: str):
    """
    Determines if a molecule is an alpha-aminoacyl group.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an alpha-aminoacyl group, False otherwise
        str: Reason for classification
    """
    # Replace * with dummy atom [X] for RDKit compatibility
    smiles = smiles.replace('*', '[X]')
    
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Check for dummy atom (attachment point)
    dummy_atoms = [atom for atom in mol.GetAtoms() if atom.GetAtomicNum() == 0]
    if len(dummy_atoms) != 1:
        return False, "Must have exactly one attachment point (*)"
        
    dummy_atom = dummy_atoms[0]
    
    # Check for C(=O)- group attached to dummy atom
    for neighbor in dummy_atom.GetNeighbors():
        if neighbor.GetSymbol() != 'C':
            continue
            
        carbonyl = False
        alpha_carbon = None
        
        for carbonyl_neighbor in neighbor.GetNeighbors():
            if carbonyl_neighbor.GetSymbol() == 'O' and carbonyl_neighbor.GetTotalNumHs() == 0:
                # Found C=O
                carbonyl = True
            elif carbonyl_neighbor.GetSymbol() == 'C':
                # Found potential alpha carbon
                alpha_carbon = carbonyl_neighbor
                
        if not carbonyl or alpha_carbon is None:
            continue
            
        # Check alpha carbon has NH2 group
        amine_found = False
        for alpha_neighbor in alpha_carbon.GetNeighbors():
            if alpha_neighbor.GetSymbol() == 'N' and alpha_neighbor.GetTotalNumHs() >= 1:
                amine_found = True
                break
                
        if amine_found:
            # Check stereochemistry if present
            stereo = ""
            if alpha_carbon.GetChiralTag() == Chem.ChiralType.CHI_TETRAHEDRAL_CCW:
                stereo = "L-"
            elif alpha_carbon.GetChiralTag() == Chem.ChiralType.CHI_TETRAHEDRAL_CW:
                stereo = "D-"
                
            # Determine side chain
            side_chain = []
            for alpha_neighbor in alpha_carbon.GetNeighbors():
                if alpha_neighbor.GetSymbol() not in ['N', 'C'] or \
                   (alpha_neighbor.GetSymbol() == 'C' and alpha_neighbor != neighbor):
                    side_chain.append(alpha_neighbor.GetSymbol())
                    
            return True, f"{stereo}alpha-aminoacyl group with side chain containing {', '.join(side_chain)}"
            
    return False, "No alpha-aminoacyl group found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22487',
                          'name': 'alpha-aminoacyl group',
                          'definition': 'A univalent carboacyl group formed by '
                                        'loss of OH from the carboxy group of '
                                        'an alpha-amino acid.',
                          'parents': ['CHEBI:27207']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183868,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673689591786}


"""
Classifies: CHEBI:22944 butanediols
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_butanediols(smiles: str):
    """
    Determines if a molecule is a butanediol or a derivative of a butanediol.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a butanediol or a derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule has 4 carbon atoms
    if Descriptors.HeavyAtomCount(mol) != 6:
        return False, "Molecule does not have 6 heavy atoms (4 carbons + 2 oxygens)"

    # Check if the molecule has 2 oxygen atoms
    oxygen_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if oxygen_count != 2:
        return False, "Molecule does not have 2 oxygen atoms"

    # Check if the oxygen atoms are part of alcohol groups
    alcohols = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 1]
    if len(alcohols) != 2:
        return False, "Molecule does not have 2 alcohol groups"

    # Check if the alcohol groups are separated by 2 or 3 carbon atoms
    for alcohol1, alcohol2 in ((alcohols[0], alcohols[1]), (alcohols[1], alcohols[0])):
        path = Chem.GetShortestPath(mol, alcohol1.GetIdx(), alcohol2.GetIdx())
        if len(path) in [3, 4]:
            return True, "Molecule is a butanediol or a derivative"

    return False, "Molecule is not a butanediol or a derivative"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22944',
                          'name': 'butanediols',
                          'definition': 'A diol that is a butanediol or a '
                                        'derivative of a butanediol.',
                          'parents': ['CHEBI:23824']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 9,
    'num_true_negatives': 183908,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.1,
    'recall': 1.0,
    'f1': 0.18181818181818182,
    'accuracy': 0.9999510651485989}


"""
Classifies: CHEBI:25701 organic oxide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_organic_oxide(smiles: str):
    """
    Determines if a molecule is an organic oxide (oxygen bonded to carbon).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an organic oxide, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule from SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find all oxygen atoms
    oxygen_atoms = [atom for atom in mol.GetAtoms() if atom.GetAtomicNum() == 8]
    if not oxygen_atoms:
        return False, "No oxygen atoms found"

    # Check each oxygen atom for carbon neighbors
    organic_oxide_oxygens = []
    for oxygen in oxygen_atoms:
        neighbors = [neighbor for neighbor in oxygen.GetNeighbors()]
        carbon_neighbors = [n for n in neighbors if n.GetAtomicNum() == 6]
        
        if carbon_neighbors:
            # Get bonding pattern
            bond_types = []
            for neighbor in neighbors:
                bond = mol.GetBondBetweenAtoms(oxygen.GetIdx(), neighbor.GetIdx())
                bond_types.append(bond.GetBondType())
            
            neighbor_elements = [n.GetSymbol() for n in neighbors]
            organic_oxide_oxygens.append((neighbor_elements, bond_types))

    if organic_oxide_oxygens:
        # Create detailed description of bonding patterns
        patterns = []
        for neighbors, bonds in organic_oxide_oxygens:
            pattern = f"O bonded to {' and '.join(neighbors)}"
            patterns.append(pattern)
        
        return True, f"Organic oxide found with patterns: {'; '.join(patterns)}"
    
    return False, "No oxygen atoms bonded to carbon found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25701',
                          'name': 'organic oxide',
                          'definition': 'An oxide in which the oxygen atom is '
                                        'bonded to a carbon atom.',
                          'parents': ['CHEBI:25741', 'CHEBI:72695']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 5,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.09090909090909091}


"""
Classifies: CHEBI:136838 2-methoxyethyl ester
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_2_methoxyethyl_ester(smiles: str):
    """
    Determines if a molecule is a 2-methoxyethyl ester.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a 2-methoxyethyl ester, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find all ester bonds
    ester_bonds = []
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.ESTERS:
            ester_bonds.append(bond)

    if not ester_bonds:
        return False, "No ester bonds found"

    # Check if the ester is a 2-methoxyethyl ester
    for bond in ester_bonds:
        atom1 = mol.GetAtomWithIdx(bond.GetBeginAtomIdx())
        atom2 = mol.GetAtomWithIdx(bond.GetEndAtomIdx())

        # Check if one of the atoms is an oxygen
        if atom1.GetSymbol() == 'O':
            ester_oxygen = atom1
            ester_carbon = atom2
        elif atom2.GetSymbol() == 'O':
            ester_oxygen = atom2
            ester_carbon = atom1
        else:
            continue

        # Check if the carbon atom is connected to a methoxyethyl group
        methoxyethyl_found = False
        for neighbor in ester_carbon.GetNeighbors():
            if neighbor.GetSymbol() == 'C':
                neighbor_neighbors = [mol.GetAtomWithIdx(idx) for idx in neighbor.GetNeighborIds()]
                if any(atom.GetSymbol() == 'O' and len(atom.GetNeighbors()) == 2 for atom in neighbor_neighbors):
                    methoxyethyl_found = True
                    break

        if methoxyethyl_found:
            return True, "The molecule is a 2-methoxyethyl ester"

    return False, "The molecule is not a 2-methoxyethyl ester"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:136838',
                          'name': '2-methoxyethyl ester',
                          'definition': 'A carboxylic ester resulting from the '
                                        'formal condensation between a '
                                        'carboxylic acid and the hydroxy group '
                                        'of 2-methoxyethanol. In contrast to '
                                        'many other water-solubilising esters, '
                                        'the 2-methoxyethyl esters of many '
                                        'amino acids are crystalline, allowing '
                                        'them to be easily purified.',
                          'parents': ['CHEBI:25698', 'CHEBI:33308']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "type object 'BondType' has no attribute 'ESTERS'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:26822 sulfide
"""
from rdkit import Chem
from rdkit.Chem import Descriptors

def is_sulfide(smiles: str):
    """
    Determines if a molecule is a sulfide, defined as any sulfur molecular entity
    that involves either covalently bonded or anionic sulfur.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a sulfide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of sulfur atoms
    if not any(atom.GetSymbol() == 'S' for atom in mol.GetAtoms()):
        return False, "No sulfur atoms present"

    # Check for covalently bonded or anionic sulfur
    sulfur_atoms = [atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'S']
    for sulfur_atom in sulfur_atoms:
        if sulfur_atom.GetFormalCharge() == -1:
            return True, "Molecule contains anionic sulfur"
        elif any(bond.GetBondType() == Chem.BondType.SINGLE for bond in sulfur_atom.GetBonds()):
            return True, "Molecule contains covalently bonded sulfur"

    return False, "No covalently bonded or anionic sulfur found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26822',
                          'name': 'sulfide',
                          'definition': 'Any sulfur molecular entity that '
                                        'involves either covalently bonded or '
                                        'anionic sulfur.',
                          'parents': ['CHEBI:26835']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 72,
    'num_false_positives': 100,
    'num_true_negatives': 676,
    'num_false_negatives': 2,
    'num_negatives': None,
    'precision': 0.4186046511627907,
    'recall': 0.972972972972973,
    'f1': 0.5853658536585366,
    'accuracy': 0.88}


"""
Classifies: CHEBI:16412 lipopolysaccharide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_lipopolysaccharide(smiles: str):
    """
    Determines if a molecule is a lipopolysaccharide based on structural features.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a lipopolysaccharide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for minimum size/complexity
    num_atoms = mol.GetNumAtoms()
    if num_atoms < 20:
        return False, "Too small to be a lipopolysaccharide"

    # Check for presence of sugar units (cyclic ethers)
    sugar_pattern = Chem.MolFromSmarts('[CR1]1[CR1][CR1][CR1][CR1]O1')
    if not mol.HasSubstructMatch(sugar_pattern):
        return False, "No sugar/saccharide units found"
    
    sugar_matches = len(mol.GetSubstructMatches(sugar_pattern))
    if sugar_matches < 1:
        return False, "Insufficient number of sugar units"

    # Check for hydroxy groups on sugars
    hydroxy_pattern = Chem.MolFromSmarts('[CR1]([CR1])O')
    if not mol.HasSubstructMatch(hydroxy_pattern):
        return False, "No hydroxy groups found on sugar units"

    # Check for fatty acid components
    fatty_acid = Chem.MolFromSmarts('C(=O)O')
    if not mol.HasSubstructMatch(fatty_acid):
        return False, "No fatty acid components found"
    
    # Check for long alkyl chains
    alkyl_chain = Chem.MolFromSmarts('CCCCC')
    if not mol.HasSubstructMatch(alkyl_chain):
        return False, "No long alkyl chains found"

    # Check for glycosidic linkages
    glycosidic = Chem.MolFromSmarts('[CR1]O[CR1]')
    if not mol.HasSubstructMatch(glycosidic):
        return False, "No glycosidic linkages found"

    # If all criteria are met, classify as lipopolysaccharide
    reason = f"Contains {sugar_matches} sugar units with hydroxy groups, fatty acid components, and glycosidic linkages"
    return True, reason


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16412',
                          'name': 'lipopolysaccharide',
                          'definition': 'Liposaccharide natural compounds '
                                        'consisting of a trisaccharide '
                                        'repeating unit (two heptose units and '
                                        'octulosonic acid) with '
                                        'oligosaccharide side chains and '
                                        '3-hydroxytetradecanoic acid units '
                                        '(they are a major constituent of the '
                                        'cell walls of Gram-negative '
                                        'bacteria).',
                          'parents': ['CHEBI:35740', 'CHEBI:65212']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 12,
    'num_false_positives': 100,
    'num_true_negatives': 1630,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.10714285714285714,
    'recall': 1.0,
    'f1': 0.19354838709677416,
    'accuracy': 0.9425947187141217}


"""
Classifies: CHEBI:189832 hydroperoxy polyunsaturated fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_hydroperoxy_polyunsaturated_fatty_acid(smiles: str):
    """
    Determines if a molecule is a hydroperoxy polyunsaturated fatty acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroperoxy polyunsaturated fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"

    # Check for hydroperoxy group (-OOH)
    hydroperoxy_pattern = Chem.MolFromSmarts('[OH]O')
    hydroperoxy_matches = mol.GetSubstructMatches(hydroperoxy_pattern)
    if not hydroperoxy_matches:
        return False, "No hydroperoxy group found"

    # Count number of double bonds
    double_bond_pattern = Chem.MolFromSmarts('C=C')
    double_bond_matches = mol.GetSubstructMatches(double_bond_pattern)
    
    if len(double_bond_matches) < 2:
        return False, "Not polyunsaturated (fewer than 2 double bonds)"

    # Verify it's a fatty acid by checking carbon chain length
    # Typically fatty acids have 12+ carbons
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 12:
        return False, "Carbon chain too short for fatty acid"

    return True, f"Contains {len(hydroperoxy_matches)} hydroperoxy group(s) and {len(double_bond_matches)} double bonds"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:189832',
                          'name': 'hydroperoxy polyunsaturated fatty acid',
                          'definition': 'Any polyunsaturated fatty acid '
                                        'carrying one or more hydroperoxy '
                                        'substituents.',
                          'parents': ['CHEBI:194321', 'CHEBI:26208']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 11,
    'num_false_positives': 17,
    'num_true_negatives': 183807,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.39285714285714285,
    'recall': 1.0,
    'f1': 0.5641025641025641,
    'accuracy': 0.9999075257703919}


"""
Classifies: CHEBI:26961 thiophenes
"""
from rdkit import Chem
from rdkit.Chem import rdchem

def is_thiophene(smiles: str):
    """
    Determines if a molecule contains at least one thiophene ring.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule contains at least one thiophene ring, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains a thiophene ring
    thiophene_ring = Chem.MolFromSmarts('c1cccs1')
    thiophene_matches = mol.GetSubstructMatches(thiophene_ring)

    if thiophene_matches:
        return True, "Molecule contains at least one thiophene ring"
    else:
        return False, "Molecule does not contain a thiophene ring"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:26961',
                          'name': 'thiophenes',
                          'definition': 'Compounds containing at least one '
                                        'thiophene ring.',
                          'parents': ['CHEBI:38106']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 33,
    'num_false_positives': 100,
    'num_true_negatives': 10697,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.24812030075187969,
    'recall': 1.0,
    'f1': 0.3975903614457831,
    'accuracy': 0.9907663896583564}


"""
Classifies: CHEBI:32957 lysophosphatidic acids
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_lysophosphatidic_acids(smiles: str):
    """
    Determines if a molecule is a lysophosphatidic acid, defined as:
    Any monoacylglycerol phosphate obtained by hydrolytic removal of one of the two acyl groups of any phosphatidic acid or derivatives therein.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a lysophosphatidic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for the presence of a phosphate group
    has_phosphate = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'P' and atom.GetTotalDegree() == 4:
            has_phosphate = True
            break

    if not has_phosphate:
        return False, "No phosphate group found"

    # Check for the presence of a glycerol moiety
    has_glycerol = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetTotalDegree() == 2:
            neighbors = [mol.GetAtomWithIdx(neighbor.GetIdx()) for neighbor in atom.GetNeighbors()]
            if len(neighbors) == 2 and all(neighbor.GetSymbol() == 'C' for neighbor in neighbors):
                carbon_neighbors = [mol.GetAtomWithIdx(neighbor.GetIdx()) for neighbor in neighbors[0].GetNeighbors() if neighbor.GetIdx() != atom.GetIdx()]
                if len(carbon_neighbors) == 2 and all(neighbor.GetSymbol() == 'C' for neighbor in carbon_neighbors):
                    has_glycerol = True
                    break

    if not has_glycerol:
        return False, "No glycerol moiety found"

    # Check for the presence of a single acyl chain
    acyl_chains = []
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.BondType.SINGLE and bond.GetBeginAtom().GetSymbol() == 'C' and bond.GetEndAtom().GetSymbol() == 'O':
            acyl_chain = []
            atom = bond.GetBeginAtom()
            while atom.GetSymbol() == 'C' and atom.GetTotalDegree() <= 3:
                acyl_chain.append(atom.GetIdx())
                neighbors = [mol.GetAtomWithIdx(neighbor.GetIdx()) for neighbor in atom.GetNeighbors() if neighbor.GetIdx() not in acyl_chain]
                if len(neighbors) == 1:
                    atom = neighbors[0]
                else:
                    break
            acyl_chains.append(acyl_chain)

    if len(acyl_chains) != 1:
        return False, "Incorrect number of acyl chains found"

    return True, "Molecule is a lysophosphatidic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:32957',
                          'name': 'lysophosphatidic acids',
                          'definition': 'Any monoacylglycerol phosphate '
                                        'obtained by hydrolytic removal of one '
                                        'of the two acyl groups of any '
                                        'phosphatidic acid or derivatives '
                                        'therein.',
                          'parents': ['CHEBI:16961']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 16,
    'num_true_negatives': 183849,
    'num_false_negatives': 7,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9998749129829446}